@@ -35,6 +35,7 @@ from trainer.io import (
     save_best_model,
     save_checkpoint,
 )
+from trainer.logger import logger
 from trainer.logging import ConsoleLogger, DummyLogger, logger_factory
 from trainer.trainer_utils import (
     get_optimizer,
@@ -182,14 +183,21 @@ class TrainerConfig(Coqpit):
         },
     )
     cudnn_enable: bool = field(default=True, metadata={"help": "Enable/disable cudnn explicitly. Defaults to True"})
+    cudnn_deterministic: bool = field(
+        default=True,
+        metadata={
+            "help": "Enable/disable deterministic cudnn operations. Set this True for better reproducibility. Defaults to True."
+        },
+    )
     cudnn_benchmark: bool = field(
         default=True,
         metadata={
-            "help": "Enable/disable cudnn benchmark explicitly. Set this False if your input size changes constantly. Defaults to False"
+            "help": "Enable/disable cudnn benchmark explicitly. Set this False if your input size change constantly. Defaults to False"
         },
     )
-    torch_seed: int = field(
-        default=54321, metadata={"help": "Seed for the torch random number generator. Defaults to 54321"}
+    training_seed: int = field(
+        default=54321,
+        metadata={"help": "Global seed for torch, random and numpy random number generator. Defaults to 54321"},
     )
 
 
@@ -234,13 +242,16 @@ class TrainerArgs(Coqpit):
         default=False,
         metadata={"help": "Skip training and only run evaluation and test."},
     )
-    gpu: int = field(default=None, metadata={"help": "GPU ID to use if ```CUDA_VISIBLE_DEVICES``` is not set. Defaults to None."})
+    gpu: int = field(
+        default=None, metadata={"help": "GPU ID to use if ```CUDA_VISIBLE_DEVICES``` is not set. Defaults to None."}
+    )
     # only for DDP
     rank: int = field(default=0, metadata={"help": "Process rank in a distributed training. Don't set manually."})
-    group_id: str = field(default="", metadata={"help": "Process group id in a distributed training. Don't set manually."})
+    group_id: str = field(
+        default="", metadata={"help": "Process group id in a distributed training. Don't set manually."}
+    )
 
 
-@dataclass
 class Trainer:
     def __init__(  # pylint: disable=dangerous-default-value
         self,
@@ -340,7 +351,11 @@ class Trainer:
             args, coqpit_overrides = self.parse_argv(args)
 
             # get ready for training and parse command-line arguments to override the model config
-            config = self.init_training(args, coqpit_overrides, config)
+            config, new_fields = self.init_training(args, coqpit_overrides, config)
+        elif args.continue_path or args.restore_path:
+            config, new_fields = self.init_training(args, {}, config)
+        else:
+            new_fields = {}
 
         # set the output path
         if args.continue_path:
@@ -354,7 +369,7 @@ class Trainer:
             os.makedirs(output_path, exist_ok=True)
 
         # copy training assets to the output folder
-        copy_model_files(config, output_path, new_fields=None)
+        copy_model_files(config, output_path, new_fields)
 
         # init class members
         self.args = args
@@ -368,16 +383,16 @@ class Trainer:
         assert self.grad_accum_steps > 0, " [!] grad_accum_steps must be greater than 0."
 
         # setup logging
-        # log_file = os.path.join(self.output_path, f"trainer_{args.rank}_log.txt")
-        # self._setup_logger_config(log_file)
-        time.sleep(1.0)  # wait for the logger to be ready
+        log_file = os.path.join(self.output_path, f"trainer_{args.rank}_log.txt")
+        self._setup_logger_config(log_file)
 
         # set and initialize Pytorch runtime
         self.use_cuda, self.num_gpus = setup_torch_training_env(
             cudnn_enable=config.cudnn_enable,
+            cudnn_deterministic=config.cudnn_deterministic,
             cudnn_benchmark=config.cudnn_benchmark,
             use_ddp=args.use_ddp,
-            torch_seed=config.torch_seed,
+            training_seed=config.training_seed,
             gpu=gpu if args.gpu is None else args.gpu,
         )
 
@@ -385,6 +400,7 @@ class Trainer:
         self.dashboard_logger, self.c_logger = self.init_loggers(
             self.args, self.config, output_path, dashboard_logger, c_logger
         )
+        self.c_logger.logger = logger
 
         if not self.config.log_model_step:
             self.config.log_model_step = self.config.save_step
@@ -490,7 +506,7 @@ class Trainer:
 
         # count model size
         num_params = count_parameters(self.model)
-        print("\n > Model has {} parameters".format(num_params))
+        logger.info("\n > Model has %i parameters", num_params)
 
         self.callbacks.on_init_end(self)
         self.dashboard_logger.add_config(config)
@@ -548,7 +564,6 @@ class Trainer:
         """
         # set arguments for continuing training
         if args.continue_path:
-            experiment_path = args.continue_path
             args.config_path = os.path.join(args.continue_path, "config.json")
             args.restore_path, best_model = get_last_checkpoint(args.continue_path)
             if not args.best_path:
@@ -564,16 +579,14 @@ class Trainer:
         # TODO: Maybe it is better to do it outside
         if len(coqpit_overrides) > 0:
             config.parse_known_args(coqpit_overrides, relaxed_parser=True)
-        experiment_path = args.continue_path
 
         # update the config.json fields and copy it to the output folder
+        new_fields = {}
         if args.rank == 0:
-            new_fields = {}
             if args.restore_path:
                 new_fields["restore_path"] = args.restore_path
             new_fields["github_branch"] = get_git_branch()
-            copy_model_files(config, experiment_path, new_fields)
-        return config
+        return config, new_fields
 
     @staticmethod
     def run_get_model(config: Coqpit, get_model: Callable) -> nn.Module:
@@ -630,18 +643,18 @@ class Trainer:
                 obj.load_state_dict(states)
             return obj
 
-        print(" > Restoring from %s ..." % os.path.basename(restore_path))
+        logger.info(" > Restoring from %s ...", os.path.basename(restore_path))
         checkpoint = load_fsspec(restore_path, map_location="cpu")
         try:
-            print(" > Restoring Model...")
+            logger.info(" > Restoring Model...")
             model.load_state_dict(checkpoint["model"])
-            print(" > Restoring Optimizer...")
+            logger.info(" > Restoring Optimizer...")
             optimizer = _restore_list_objs(checkpoint["optimizer"], optimizer)
             if "scaler" in checkpoint and self.use_amp_scaler and checkpoint["scaler"]:
-                print(" > Restoring Scaler...")
+                logger.info(" > Restoring Scaler...")
                 scaler = _restore_list_objs(checkpoint["scaler"], scaler)
         except (KeyError, RuntimeError, ValueError):
-            print(" > Partial model initialization...")
+            logger.info(" > Partial model initialization...")
             model_dict = model.state_dict()
             model_dict = set_partial_state_dict(model_dict, checkpoint["model"], config)
             model.load_state_dict(model_dict)
@@ -649,9 +662,7 @@ class Trainer:
 
         optimizer = self.restore_lr(config, self.args, model, optimizer)
 
-        print(
-            " > Model restored from step %d" % checkpoint["step"],
-        )
+        logger.info(" > Model restored from step %i", checkpoint["step"])
         restore_step = checkpoint["step"] + 1  # +1 not to immediately checkpoint if the model is restored
         restore_epoch = checkpoint["epoch"]
         torch.cuda.empty_cache()
@@ -1052,8 +1063,6 @@ class Trainer:
         if ((step + 1) % self.grad_accum_steps != 0) and (step + 1 != batch_n_steps):
             step_optimizer = False
 
-
-
         if not isinstance(self.optimizer, list):
             # training with a single optimizer
             outputs, loss_dict_new, step_time = self._optimize(
@@ -1107,8 +1116,6 @@ class Trainer:
         if step_optimizer:
             self.model.zero_grad()
 
-
-
         # update avg runtime stats
         keep_avg_update = {}
         keep_avg_update["avg_loader_time"] = loader_time
@@ -1392,11 +1399,11 @@ class Trainer:
         """Restore the best loss from the args.best_path if provided else
         from the model (`args.restore_path` or `args.continue_path`) used for resuming the training"""
         if self.restore_step != 0 or self.args.best_path:
-            print(f" > Restoring best loss from {os.path.basename(self.args.best_path)} ...")
+            logger.info(" > Restoring best loss from %s ...", os.path.basename(self.args.best_path))
             ch = load_fsspec(self.args.restore_path, map_location="cpu")
             if "model_loss" in ch:
                 self.best_loss = ch["model_loss"]
-            print(f" > Starting with loaded last best loss {self.best_loss}.")
+            logger.info(" > Starting with loaded last best loss %f", self.best_loss)
 
     def test(self, model=None, test_samples=None) -> None:
         """Run evaluation steps on the test data split. You can either provide the model and the test samples
@@ -1410,7 +1417,7 @@ class Trainer:
                 given in the initialization. Defaults to None.
         """
 
-        print(" > USING TEST SET...")
+        logger.info(" > USING TEST SET...")
         self.keep_avg_eval = KeepAverage()
 
         if model is not None:
@@ -1659,29 +1666,21 @@ class Trainer:
         return target_avg_loss
 
     def _setup_logger_config(self, log_file: str) -> None:
-        """Write log strings to a file and print logs to the terminal.
-        TODO: Causes formatting issues in pdb debugging."""
+        """Set up the logger based on the process rank in DDP."""
+        global logger  # pylint: disable=global-statement
+        import logging  # pylint: disable=import-outside-toplevel
 
-        class Logger(object):
-            def __init__(self, print_to_terminal=True):
-                self.print_to_terminal = print_to_terminal
-                self.terminal = sys.stdout
-                self.log_file = log_file
+        handler = logging.FileHandler(log_file)
+        logger.addHandler(handler)
 
-            def write(self, message):
-                if self.print_to_terminal:
-                    self.terminal.write(message)
-                with open(self.log_file, "a", encoding="utf-8") as f:
-                    f.write(message)
-
-            def flush(self):
-                # this flush method is needed for python 3 compatibility.
-                # this handles the flush command by doing nothing.
-                # you might want to specify some extra behavior here.
-                pass
-
-        # don't let processes rank > 0 write to the terminal
-        sys.stdout = Logger(self.args.rank == 0)
+        # only log to a file if rank > 0
+        if self.args.rank > 0:
+            handler = logging.FileHandler(log_file)
+            handler.setFormatter(logging.Formatter(""))
+            new_logger = logging.getLogger("trainer_ddp")
+            new_logger.addHandler(handler)
+            new_logger.setLevel(logging.INFO)
+            logger = new_logger
 
     @staticmethod
     def _is_apex_available() -> bool:
